c     jmkflt - read in and interpolate a filter
      subroutine jmkflt( damping, filename, fltidx )

      implicit none
      include 'params.for'
      external lenact
      integer   lenact
      external initje

      double precision damping(*)
      character*256  filename
      double precision fltidx

      external jmkflt_parse
      integer jmkflt_parse

      external jmkpath
      character*256 jmkpath, buf
      integer i, ifirst, ilast, line, nzero

      double precision energy0, fact0
      double precision energy1, fact1

      integer lun
      parameter ( lun = 97 )

c     the value of fltidx indicates how file should be interpreted.
c     if fltidx = 0, file contains the filter.  if fltidx > 0,
c     file is an rdb table which contains a list of index-filename
c     pairs; fltidx is then the index of the requested file

      if ( fltidx .gt. 0 ) then
         filename = jmkpath( filename, fltidx )
         if ( filename .eq. ' ' ) goto 903
      endif

      call xwrite('jmkflt: reading file ' // filename, 11)
      open (unit = lun, file = filename, status = 'old', err=901)

c     scan filter file, interpolating onto the energy bins in
c     the spectrum.  do it line by line to avoid having to read
c     the whole thing into memory.

c     read past any comments at beginning of file
      line = 1
 10   read( lun, '(a)', end=902 ) buf
      line = line + 1
      if ( buf(1:1) .eq. '#' ) goto 10

c     read second line in two line header. it isn't used, so ignore it
      read( lun, '(a)', end=902 ) buf
      line = line + 1

c     determine first valid energy; damping values for lower energies
c     will be set to 0
      read(lun, '(a)', end = 902, err = 905) buf
      if ( jmkflt_parse( buf, energy0, fact0 ) .ne. 0 ) goto 905
      line = line + 1

      nzero = 0
      ifirst = 1
      do 20 i = 1, nInt
         if ( evArr(i) .ge. energy0 ) goto 30
         damping(i) = 0.0
         nzero = nzero + 1
         ifirst = i + 1
 20   continue

c     if this statement is reached, the filter is beyond the energies
c     in the spectrum, which is kind of weird
      call xwrite( 'jmkflt: no matches in energy with ' // filename, 1)
      close(lun)
      goto 903
      
c     pre-read next filter entry to set up range for interpolation
 30   read(lun, '(a)', end = 902, err=905) buf
      if ( jmkflt_parse( buf, energy1, fact1 ) .ne. 0 ) goto 905
      line = line + 1

c     loop over spectrum, reading in new filter values as necessary
      ilast = i
      do 40 i = ifirst, nInt

c     if the spectrum energy is outside of the current filter range,
c     read enough of the filter to establish a bounding range.
         if ( evArr(i) .gt. energy1 ) then
 35         energy0 = energy1
            fact0 = fact1
            read(lun, '(a)', end=41, err=905) buf
            if ( jmkflt_parse( buf, energy1, fact1 ) .ne. 0 ) goto 905
            line = line + 1
            if ( evArr(i) .gt. energy1 ) goto 35
         endif

c     linearly interpolate damping coefficient
         damping(i) =  fact0 + (fact1 - fact0) * 
     +        (evArr(i) - energy0)/(energy1 - energy0)
         if ( damping(i) .eq. 0.0 ) nzero = nzero + 1
         ilast = ilast + 1
 40   continue
 41   continue

c     check that there was some overlap between the filter and the
c     spectrum
      if ( ilast .eq. ifirst ) then

         call xwrite( 'jmkflt: no matches in energy with ' // filename,
     $        1)
         close(lun)
         goto 903
      endif

c     if the entire spectrum hasn't been processed, this implies
c     that the filter has cut off early.  set the remaining damping
c     values to zero.
      if ( ilast .le. nInt ) then
         do 80 i = ilast, nInt
            nzero = nzero + 1
            damping(i) = 0.0
 80      continue
      endif

      close(lun) 

      if ( nzero .eq. nInt ) then
         call xwrite( 'jmkflt: All filter values interploated from ' //
     $        filename(:lenact(filename)) // ' were 0.0!!', 1)
      endif

      return

c     error opening file
 901  call xwrite( 'jmkflt: error opening ' // filename, 1 )
      goto 903

c     unexpected eof
 902  call xwrite( 'jmkflt: unexpected EOF on ' // filename, 1 )
      close(lun)
      goto 903

c     error whilst reading
 905  write( buf,
     $     '(''jmkflt: error in '',a,'' at line '',i)' )
     $     filename(:lenact(filename)), line
      call xwrite( buf, 1 )
      close(lun)
      goto 903

c     an error has ocurred. set the damping to simulate a
c     pass-through filter
 903  do 904 i = 1, nInt
         damping(i) = 1.0
 904  continue
      return
      
      end

      function jmkflt_parse( buf, energy, trans )

      character *(*) buf
      double precision energy, trans

      integer i

      character*1 tab
      parameter ( tab = '\t' )

      jmkflt_parse = 1

      i = index( buf, tab )
      if ( i .eq. 0 ) return

      read( buf(1:i-1), *, err = 900 ) energy

      buf = buf(i+1:)
      i = index( buf, tab )
      if ( i .eq. 0 ) i = len(buf)
      read( buf(1:i-1), *, err = 900 ) trans

      jmkflt_parse = 0
      return

c     bad rdb row
 900  return
      end
