cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       Programmer: Eugene Tsiang                                      c
c       AXAF Mission Support Team                                      c
c       Date: 8/26/96                                                  c
c       Place: Smithsonian Astrophysical Observatory, Cambridge, MA    c
c       Spectral Analysis Program for XSPEC Revision 6.00              c
c       Module name: jmkmod.f
c       Modifications
c       8/30/96 distinction between random and periodic pulsers 
c               routines ranpile and perpile replace pilesim 
c       9/4/96  correction to periodic pulser
c       modified 10/9/96, added synchrotron
c       modified 10/25/96, corrected wrong normalization in periodic
c             pulser
c       eliminated dparam, filnum 12/23/96, multiline capability
c       added version control 12/23/96, 
c       corrected mistake in last virtual spectral bin (wrong index) 
c       version 6.4
c       version 6.5 added ice filter 12/30/96
c       added HRMA parameters and abswitch 12/31/96
c       increased number of filters to 8 12/31/96, or any number
c       implemented non-normalization of continuum for abswitch 1/2/97
c       implemented HRMA effective area 1/6/97
c       Version 7.0 released 1/6/97 
c	version 7.2 moved falling tail; added lead filter; added table filters
c		released 3/20/1998
c       version 7.32 fixed obscure bug in sglshelf; pileup width calculation
c                    now tracks actual electronics; continuum bug fixed;
c                    external filters; multiple optical constant dbs;
c	version 7.33 small bug in lmodel.dat fixed
c	version 7.4 xspec 11 compatibility
c	version 7.41 minor common block cleanup
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       Modifications were made to implement the shift Laplace factor
c       after the pileup calculations. Originally, the shift was
c       implemented before the pileup calculations. This placed 
c       the pileup features at wrong channel numbers.
c       SRS 2/24/1998
c
c       Similar modifications were made to implement the broadening
c       Laplace factor after the pileup calculations. This was done
c       to get the correct width for the pileup peaks. That is, the
c       quadratic sum of 2 fano broadenings and only one electronic
c       broadening.
c       SRS 6/26/98
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine jmkmod(ear, ne, param, ifl, photar, photer)
      integer  ifl, ne
      real*4 ear(0:ne), param(*), photar(ne), photer(ne)
      real*4 grit
      integer*4 i, intdummy

      double precision contpdf(8192)
      double complex mgfarr(8192)
      
      integer numcomp, numfilter, tally

      integer maxcomp, maxfilter, intdum
      parameter (maxcomp = 16, maxfilter = 16)
c      increased number of components and filters 12/23/96

      double precision     
     1    jfano, jpolyah, jeoff, jionen, jgain, jshift, jsigma, jdgain, 
     2    jnchan, jsestype, 
     3    jcontinuum, jecont, jcontnorm, jemax, jemin, jechar, jewidth, 
     4    jnInt, jtemper, jpressure, jargrat,
     5    jshelfswitch, jt1norm, jt1par, jt2norm, jt2par, jshelfnorm, 
     6    jpulser, jpulsepos, jpulsesigma, jpulsenorm,
     7    jnumcomp, jnumfilter, jpileup, jpilepar,
     8    jbfield, jcurrent, jincline,
     9    jabswitch, jphi, jtheta, joptconst,
     1    jlinenergy(maxcomp), jlinenorm(maxcomp),
     2    jsrcstack(maxfilter), jsrcthick(maxfilter)
           
c singlelimit + 2*maxcomp + 2*maxfilter parameters 12/23/96 eyt      
c passed to jmkply
	
*      character*79 dummy
*      write(dummy,800)ear(0),ear(ne),(param(i),i=1,5)
*800   format('JMmodel: ',1p,7g10.2)
*	call xwrite(dummy,25)

c version control
      character*96 dummy
      real*4 version
      version = 7.4
      write(dummy, 800) version
800   format(' 3/20/98, JMKMOD version: ', f10.2)
c      print *, dummy
      call xwrite(dummy,11)
c  uncomment this last statement within xspec

c now give them their assigned meanings and convert to dbl precision

      jfano = param(1)
      jpolyah = param(2)
      jeoff = param(3)
      jionen = param(4)
      jgain = param(5)
      jshift = param(6)
      jsigma = param(7)
      jdgain = param(8)
      jnchan = param(9)
      jsestype = param(10)
      jcontinuum = param(11)
      jecont = param(12)
      jcontnorm = param(13)
      jemax = param(14)    
      jemin = param(15)
      jechar = param(16)
      jewidth = param(17)
      jnInt = param(18)
      jtemper = param(19)
      jpressure = param(20)
      jargrat = param(21)
      jshelfswitch = param(22)
      jt1norm = param(23)
      jt1par = param(24)
      jt2norm = param(25)
      jt2par = param(26)
      jshelfnorm = param(27)
      jpulser = param(28)
      jpulsepos = param(29)
      jpulsesigma = param(30)
      jpulseNorm = param(31)
      jnumcomp = param(32)
      jnumfilter = param(33)
      jpileup = param(34)
      jpilepar = param(35)
      jbfield = param(36)
      jcurrent = param(37)
      jincline = param(38)
      jabswitch = param(39)
      jphi = param(40)
      jtheta = param(41)
      joptconst = param(42)
          
      numcomp = jnumcomp
      tally = 42
c singlelimit must be changed in params.for AND here, 1/2/97

      intdum = tally + numcomp
      
      do 2000 i = 1, numcomp
          jlinenergy(i) = param(tally +i)
          jlinenorm(i) = param(intdum + i)
 2000 continue
 
      numfilter = jnumfilter      
      tally = tally + 2*numcomp
      intdum = tally + numfilter
      
      do 3000 i = 1,numfilter
          jsrcstack(i) = param(tally + i)
          jsrcthick(i) = param(intdum + i)
 3000 continue          
      
*      print *, 'In jmkmod, jsrcstack = ', jsrcstack
*      print *, 'In jmkmod, jsrcthick = ', jsrcthick 
*      
      tally = tally + 2*numfilter
*      print *, 'In jmkmod, the total number of parameters is ', tally
       
            
      call jmkply(contpdf, mgfarr,
     1    jfano, jpolyah, jeoff, jionen, jgain, jshift, jsigma, jdgain, 
     2    jnchan, jsestype, 
     3    jcontinuum, jecont, jcontnorm, jemax, jemin, jechar, jewidth, 
     4    jnInt, jtemper, jpressure, jargrat,
     5    jshelfswitch, jt1norm, jt1par, jt2norm, jt2par, jshelfnorm, 
     6    jpulser, jpulsepos, jpulsesigma, jpulsenorm, 
     7    jnumcomp, jnumfilter, jpileup, jpilepar, 
     8    jbfield, jcurrent, jincline,
     9    jabswitch, jphi, jtheta, joptconst,
     1    jlinenergy, jlinenorm, jsrcstack, jsrcthick)

      GRIT = 1.d-8
           
**      print *, 'In jmkmod, '
      DO 10, I = 1, NE
         INTDUMMY = 0.5*((EAR(I-1)+EAR(I))) + GRIT
         PHOTAR(I) = contpdf(INTDUMMY)
**         print *, intdummy, photar(i)
   10 CONTINUE

c photar is the returned array, ear, ne, param are input
      return
      END

      subroutine jmkply(contpdf, mgfarr,
     1    jfano, jpolyah, jeoff, jionen, jgain, jshift, jsigma, jdgain, 
     2    jnchan, jsestype, 
     3    jcontinuum, jecont, jcontnorm, jemax, jemin, jechar, jewidth, 
     4    jnInt, jtemper, jpressure, jargrat,
     5    jshelfswitch, jt1norm, jt1par, jt2norm, jt2par, jshelfnorm, 
     6    jpulser, jpulsepos, jpulsesigma, jpulsenorm, 
     7    jnumcomp, jnumfilter, jpileup, jpilepar, 
     8    jbfield, jcurrent, jincline,
     9    jabswitch, jphi, jtheta, joptconst,
     1    jlinenergy, jlinenorm, jsrcstack, jsrcthick)
          
      include 'params.for'
      external initje

      double precision contpdf(tChan)
      double complex mgfarr(tChan)

      double precision     
     1    jfano, jpolyah, jeoff, jionen, jgain, jshift, jsigma, jdgain, 
     2    jnchan, jsestype, 
     3    jcontinuum, jecont, jcontnorm, jemax, jemin, jechar, jewidth, 
     4    jnInt, jtemper, jpressure, jargrat,
     5    jshelfswitch, jt1norm, jt1par, jt2norm, jt2par, jshelfnorm, 
     6    jpulser, jpulsepos, jpulsesigma, jpulsenorm,
     7    jnumcomp, jnumfilter, jpileup, jpilepar,
     8    jbfield, jcurrent, jincline,
     9    jabswitch, jphi, jtheta, joptconst,
     1    jlinenergy(*), jlinenorm(*), jsrcstack(*), jsrcthick(*)
     
c total of singlelimit + 2*maxcomp + 2*maxfilter parameters 10/17/96 eyt      
c singlelimit = 42 12/31/96

      double complex jmmgf, contply, dgjmgf, contdg, 
     1      binmgf, contexp, brshft
      external jmmgf, contply, dgjmgf, contdg, binmgf, contexp, brshft

      integer k, tally

      fano = jfano
      polyaH = jpolyah
      eOff = jeOff
      ionen = jionen
      gain = jgain
      shift = jshift
      sigma = jsigma
      dgain = jdgain
      nchan = jnchan
      sestype = jsestype
      continuum = jcontinuum
      econt = jecont
      contnorm = jcontnorm
      emax = jemax
      emin = jemin
      echar = jechar
      ewidth = jewidth
      nInt = jnInt
      temper = jtemper
      pressure = jpressure
      argrat = jargrat
      shelfswitch = jshelfswitch
      t1norm = jt1norm
      t1par = jt1par
      t2norm = jt2norm
      t2par = jt2par
      shelfnorm = jshelfnorm
      pulser = jpulser
      pulsepos = jpulsepos
      pulsesigma = jpulsesigma
      pulsenorm = jpulsenorm
      numcomp = jnumcomp
      numfilter = jnumfilter
      pileup = jpileup
      pilepar = jpilepar      
      bfield = jbfield
      current = jcurrent
      incline = jincline
      abswitch = jabswitch
      phi = jphi
      theta = jtheta
      optconst = joptconst
c strip parameters from call and put in common blocks
      
      do 10 k = 1, numcomp
          linenergy(k) = jlinenergy(k)
          linenorm(k) = jlinenorm(k)
   10 continue
   
      do 20 k = 1, numfilter
          srcstack(k) = jsrcstack(k)
          srcthick(k) = jsrcthick(k)
   20 continue
             
c end source stack      

           tChan = nChan*2
           tChan1 = tChan + two
           nInt1 = nInt - 1
           deltaE = 1.d0
           piDelt = dcmplx(0.d0, - mypi/(dble(nChan)*deltaE))
           dEmax = (eMax - eMin)/ nInt1

      if (eMax.ne.eMaxOld.or.
     1	eMin.ne.eMin.or.
     2  dEmax.ne.dEmaxOld) then
           enerflag = 1
c this means that some energy parameters have changed
           eMaxOld = eMax
           eMinOld = eMin
           dEmaxOld = dEmax
           do 200 k = 1, nInt 
                eArray(k) = eMin + dEmax*(k - 1)
                evArr(k) = 1000.*eArray(k)
  200      continue    
      else
           enerflag = 0
c this means that no energy parameters have changed
      endif

      tally = singlelimit + 2*numcomp + 2*numfilter
*      print *, 'In jmkply, the total number of parameters is ', tally
*      
*      print *, 'In jmkply', 
*     1' fano = ', fano, ' polyaH = ', polyah, ' eoff = ', eoff,
*     2' ionEn = ', ionEn, ' gain = ', gain, ' shift = ', shift,
*     3' sigma = ', sigma, ' dgain = ', dgain, ' nchan = ', nchan,
*     4' sestype = ', sestype, ' continuum = ', continuum,
*     5' cont = ', econt, ' emax = ', emax, ' emin = ', emin,
*     6' echar = ', echar, ' ewidth = ', ewidth, ' nInt = ', nInt,
*     7' temperature = ', temper, 'pressure = ', pressure,
*     8' argon ratio = ', argrat, ' shelfswitch = ', shelfswitch,
*     9' t1norm = ', t1norm, ' t1par = ', t1par, ' t2norm = ', t2norm,
*     1' t2par = ', t2par, ' shelfnorm = ', shelfnorm, 
*     2' pulser = ',pulser, ' pulsepos = ', pulsepos, 
*     3' pulsesigma = ',pulsesigma, 
*     4' number of components = ',numcomp,
*     5' numfilter = ', numfilter, ' pileup = ', pileup,
*     6' pilepar = ', pilepar, ' line energies = ', linenergy,
*     7' linenorm = ', linenorm, ' filter stack = ', srcstack,
*     8' filter thicknesses = ', srcthick,
*     9' bfield = ', bfield, 'current = ', current,
*     1' incline = ', incline, ' abswitch = ', abswitch,
*     2' phi = ', phi, ' theta = ', theta, ' optconst = ', optconst 
    
*      print *, 'In jmkply, auxiliaries: ', ' tChan = ', tChan, 
*     1    ' tChan1 = ', tChan1, ' nInt1 = ', nInt1,' deltaE = ',deltaE,
*     2    ' piDelt = ', piDelt, ' dEmax = ', dEmax 
*
**      print *, 'eArray in jmkply '
**      do 260 k = 1, nInt
**      print *, eArray(k), ' ', evArr(k)
**  260 continue

      if (continuum.ne.0) call damper
c damper returns an array 'alldamp(nInt)' containing the
c action of all the sources(filters)

c single electron spectral type sestype selected:
      if (sestype.eq.2) then
          call mgfmake(binmgf, contexp, mgfarr)
      else if (sestype.eq.3) then
          call mgfmake(dgjmgf, contdg, mgfarr)          
      else
          call mgfmake(jmmgf, contply, mgfarr)
c this is the default (sestype.eq.1)          
      endif
c pileup if any is performed within mgfmake 8/30/96

**      print *, 'pileup parameter in jmkply ', pileup, pilepar

      call invert(mgfarr, contpdf)
c jmpdf(tChan) returned by Fourier inverting mgfarr(tChan)

**      print *, 'In jmkply ' 

**      do 2000 k = 1,tChan
**          print *, contpdf(k), mgfarr(k)
** 2000 continue

      return
      end

      subroutine ranpile(mgfarr)
c for random pulsers
      include 'params.for'
      external initje
      double complex mgfarr(tChan)
      integer i
      double precision someval, cpilepar, normsum
      double complex mgfdum
      
      if (pilepar.eq.one) return
      
      if (pilepar.eq.0.0) then
c almost never happens, because this implies unlimited pileup
          do 800 i = 1,tChan
              mgfarr(i) = (0.0,0.0)
800   continue
      return
      endif              

      normsum = mgfarr(1)                      
      cpilepar = one - pilepar
      someval = cpilepar/normsum

      do 400 i = 1, tChan
          mgfdum = mgfarr(i)
          mgfarr(i) = pilepar*mgfdum/(one - someval*mgfdum) 
  400 continue
      
      return
      end                              

      subroutine perpile(mgfarr, pulsarr)
c for periodic pulsers, mgfarr not normalized
      include 'params.for'
      external initje
      double complex mgfarr(tChan), pulsarr(tChan)
      integer i
      double precision someval, cpilepar, normsum
      double complex mgfdum

      if (pilepar.eq.one) return

      if (pilepar.eq.0.0) then
c almost never happens, because this implies unlimited pileup
           do 800 i = 1, tChan
               mgfarr(i) = (0.0, 0.0)  
800        continue            
      return
      endif

      normsum = mgfarr(1)
      cpilepar = one - pilepar
      someval = cpilepar/normsum

      do 400 i = 1, tChan
           mgfdum = mgfarr(i)
           mgfarr(i) = pilepar*(pulsarr(i) + mgfdum)/
     1                 (one - someval*mgfdum)
  400 continue

      return
      end

      subroutine mgfmake(linemgf, contmgf, mgfarr)
      
      double complex linemgf, contmgf, brshft, plsrmg
      external linemgf, contmgf, brshft, plsrmg
      include 'params.for'
      external initje
      double precision oldenergy, oldshift, oldsigma
      integer j, k, dumbcomp
      double complex dumbarr(8192), lcparr(8192,32)
c make sure these arrays are large enough to hold components
      double complex mgfarr(tChan)
      double complex dumbtemp, sum
**      double precision dumbpdf(8192)
      
c numcomp is the number of line components    
      oldenergy = eChar      
c oldenergy saves the current of eChar, while eChar 
c is being "borrowed"

      do 650 k = 1, numcomp
          eChar = linenergy(k)
          call mgftbl(linemgf,dumbarr)
**          call invert(dumbarr, dumbpdf)
          do 660 j = 1, tChan
              lcparr(j,k) = dumbarr(j)
**              print *, dumbpdf(j), dumbarr(j)
  660     continue
      
  650 continue
      dumbcomp = numcomp
c this equates dumbcomp to the number of line components

      eChar = oldenergy
c this restores eChar back to its original value 
            
      if (continuum.ne.0) then             
          call mgftbl(contmgf, dumbarr)
          dumbcomp = dumbcomp+1
                 dumbtemp = dumbarr(1)
c normalization constant for continuum 
          if (abswitch.eq.0) then
c abswitch.eq.0 (relative calibration)
          do 670 j = 1, tChan
              dumbarr(j) = dumbarr(j)/dumbtemp
              lcparr(j,dumbcomp) = dumbarr(j)
  670     continue
c case of relative calibration where just the counts matter
          else
c abswitch.ne.0 (absolute calibration)
          do 780 j = 1, tChan
              lcparr(j,dumbcomp) = dumbarr(j)
  780     continue
c no normalization for absolute calibration 1/2/97
          endif
**          call invert(dumbarr, dumbpdf)
**          print *,'In mgfmake, the cont pdf/mgf is '
**          do 770 j = 1, tChan
**              print *, dumbpdf(j), dumbarr(j)
**  770     continue              
          linenorm(dumbcomp) = contnorm
c "borrowing" linenorm for the continuum          
      endif
c at this point, dumbcomp is the number of lines and continuum,
c if any

c by convention
c pulser>0, periodic pulser
c pulser = 0, no pulser
c pulser<0, random pulser

      if (pulser.ne.0) then
c there is a pulser
          oldshift = shift
          oldsigma = sigma
          shift = pulsepos
          sigma = pulsesigma
          call mgftbl(plsrmg, dumbarr)
c the pulser is a shifted gaussian
**          call invert(dumbarr, dumbpdf)
**          print *, 'In mgfmake, the pulser/pdf/mgf is '
          dumbcomp = dumbcomp+1
          do 680 j = 1, tChan
              lcparr(j,dumbcomp) = dumbarr(j)
**              print *, dumbpdf(j), dumbarr(j)
  680     continue                  
          shift = oldshift
          sigma = oldsigma
          linenorm(dumbcomp) = pulsenorm
      endif    
c at this point, dumbcomp is the number of lines and continuum 
c and pulser, if any

c correction added 10/25/96 due to wrong normalization for
c periodic pulser

      if (pileup.ne.0.and.pulser.gt.0) then
           do 840 j = 1, tChan
           dumbarr(j) = linenorm(dumbcomp)*lcparr(j,dumbcomp)
  840      continue
           dumbcomp = dumbcomp - 1
      endif

c this is the case of pileup with a periodic pulser
c dumbcomp is lowered by 1 to exclude the pulser, which is last

           do 700 j = 1, tChan
               sum = 0.0
           do 750 k = 1, dumbcomp
               sum = sum + linenorm(k)*lcparr(j,k)
  750      continue     
           mgfarr(j) = sum
  700      continue

      if (pileup.eq.0) then
          goto 900 
c the case of NO pileup, NO further processing of mgfarr
c
c the remaining cases have pileup
      else if (pulser.gt.0) then
c the case of a periodic pulser with pileup
           call perpile(mgfarr, dumbarr)
c dumbarr is just lcparr(j,dumbcomp+1)*linenorm(dumbcomp+1),
c the mgf of the pulser. Remember that dumbcomp has been set
c to dumbcomp-1, one less because of pulser
      else
c this includes the general cases of random pulser and no pulser
c pulser.le.0
           call ranpile(mgfarr)    
      endif
c
c Now that we have the final total mgf, implement the shift and 
c broadening.
c
900   call mgftbl(brshft, dumbarr)
      do 910 j=1, tChan
         mgfarr(j) = mgfarr(j)*dumbarr(j)
910   continue

      return
      end    

