cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       Programmer: Eugene Tsiang                                      c
c       AXAF Mission Support Team                                      c
c       Date: 8/26/96                                                  c
c       Place: Smithsonian Astrophysical Observatory, Cambridge, MA    c
c       Spectral Analysis Program for XSPEC Revision 6.00              c
c       Module name:  matnu.f
c       added ice filter,  lengthened data, mu and arr arrays 12/30/96
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine newmac(kElem, extinct)

      include 'config.inc'
      include 'params.for'
      external initje

      integer kElem

*      double precision eoff, emax, demax
*	double precision keV
      integer best
*       integer nInt
      double precision mubest, evbest
*      integer best1

      double precision mu(maxoptconsts),ev(maxoptconsts)
c ev is the energy array
c mu is the extinction array calculated from Henke

      double precision extinct(nInt), aw
c extinct is the interpolated extinction array

      character*2 el

      integer i, noptconsts

*      eoff = 0.00
*      emax = 10.	
*      nInt = 512
*      demax = (emax - eoff)/(nInt - 1)
*      keV = 1000.
*       print *, ' energy array = '
*      do 650 i = 1, nInt
*	evArr(i) = keV*(eoff + (i - 1)*demax)
*        print *, i, evArr(i)
*  650 continue
     
      aw = elemWate(kElem)
      el = elemName(kElem)

      call jmkopt( aw, el, ev, mu, noptconsts )

      best = 1
*      best1 = 2

      do 660 i = 1, nInt
*      call locate(arr, n, evArr(i), best1)
         call hunta(ev, noptconsts, evArr(i), best)
         if (noptconsts .gt. best .and. best .gt. 0) then
            mubest = mu(best)
            evbest = ev(best)
            extinct(i) = mubest + (mu(best+1)-mubest)*
     1           (evArr(i) - evbest)/(ev(best+1)- evbest)
         else if ( best .le .0) then 
            extinct(i) = 999999999.
            evbest = evArr(i)
            mubest = extinct(i)
         else
            mubest = mu(best)
            evbest = ev(best)
            extinct(i) = mubest 
         endif	
 660  continue

      return
      end


      subroutine jmkopt( aw, el, ev, mu, noptconsts )

c     atomic weight
      double precision aw

c     element name
      character *2 el

c     energies read from optical constants file
      double precision ev(*)

c     calculated extinction array
      double precision mu(*)

c     number of optical constants read
      integer noptconsts
c     

      include 'config.inc'
      include 'params.for'

      external  lenact
      integer   lenact

      external initje

      external jmkpath
      character*256 jmkpath

      character*512 filename
      character*256 buf
      double precision f1, f2
      double precision energy

      integer lun
      parameter ( lun = 99 )

c     use value of optconst to determine where optical constants directory
c     is located. 

c     default directory for optical constants
      if ( optconst .eq. 0 ) then
         buf = optconstdir

c     look in the default rdb file 
      elseif ( optconst .ge. 1 .and. optconst .le. 999 ) then
         buf = jmkpath( optconsttbl, optconst )
         if ( buf .eq. ' ' ) then
            call xwrite( 'jmkopt: using default optical constants for '
     $           // el, 1 )
            buf = optconstdir
         endif

c     look in the directory environmental variable for the directory
      elseif ( optconst .eq. 1000 ) then
         call xwrite( 'jmkopt: reading environmental variable ' //
     $        optconstenvdir, 11)
         call getenv( optconstenvdir, buf )
         if ( buf .eq. ' ' ) then
            call xwrite( 'jmkopt: environmental variable ' //
     $           optconstenvdir // ' doesn''t exist', 1 )
            call xwrite( 'jmkopt: using default optical constants for '
     $           // el, 1 )
            buf = optconstdir
         endif

c     look in the rdb table pointed to by the table environmental variable
      elseif ( optconst .ge. 1001 .and. optconst .le. 1999 ) then
         call xwrite( 'jmkopt: reading environmental variable ' //
     $        optconstenvtbl, 11)
         call getenv( optconstenvtbl, buf )
         if ( buf .eq. ' ' ) then
            call xwrite( 'jmkopt: environmental variable ' //
     $           optconstenvtbl // ' doesn''t exist', 1 )
            call xwrite( 'jmkopt: using default optical constants for '
     $           // el, 1 )
            buf = optconstdir
         else
            buf = jmkpath( buf, optconst - 1000 )
            if ( buf .eq. ' ' ) then
               call xwrite( 'jmkopt: using default optical constants',
     $              1 )
               buf = optconstdir
            endif
         endif
         
c     look in the directory environmental variable for the directory
c     if the environmental variable exists, else use the default
      elseif ( optconst .eq. 2000 ) then
         call xwrite( 'jmkopt: reading environmental variable ' //
     $        optconstenvdir, 11)
         call getenv( optconstenvdir, buf )
         if ( buf .eq. ' ' ) then
            buf = optconstdir
         endif


c     everything else is an error; use the default directory
      else
         call xwrite( 'jmkopt: illegal optical constant directory id',
     $        1)
         call xwrite( 'jmkopt: using default optical constants for '
     $        // el, 1 )
         buf = optconstdir

      endif

      buf(lenact(buf)+1:) = '/'
      write( filename, optconstfmt ) buf(:lenact(buf)),
     $     el(:lenact(el))

      call xwrite('jmkopt: reading file ' // filename, 11)
      open( unit=lun, file = filename, status = 'old', err = 901)

c     first line is header; we skip it
      read( lun, *, end = 902 ) buf

      i = 0
 20   i = i + 1
      read ( lun, *, end = 30, err = 903) energy, f1, f2
      if ( i .gt. maxoptconsts ) then
         write( buf,
     $      '(''jmkopt: too many energies in '', a, ''; max of '', i)')
     $      filename, maxoptconsts
         call xwrite( buf, 1 )
         close(lun)
         return
      endif

c     must use temporary to avoid going past end of array if too
c     many values in file
      ev(i) = energy
      mu(i) = 4.208d7 * f2 / energy / aw
      goto 20

 30   close( lun )

      noptconsts = i - 1

      return


c     error opening file
 901  call xwrite( 'jmkopt: error opening ' // filename, 1 )
      return

c     unexpected eof
 902  call xwrite( 'jmkopt: unexpected EOF on ' // filename, 1 )
      close(lun)
      return


c     error on read
 903  write( buf,
     $     '(''jmkopt: error at line '',i, ''in '',a )' )
     $     i, filename(:lenact(filename))
      call xwrite( buf, 1 )
      close(lun)
      return

      end
