cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c	Programmer: Eugene Tsiang                                      c
c	AXAF Mission Support Team                                      c
c	Date: 8/26/96                                                  c
c	Place: Smithsonian Astrophysical Observatory, Cambridge, MA    c
c       Spectral Analysis Program for XSPEC Revision 6.00              c
c	Module name: source.f                                          c
c	Modifications:
c       9/11/96     Added Mn and polypro
c       9/16/96     Checked shelves, found error in Ge dens in ssdgerm
c       9/17/96     Changed P10 and CH4 using right formulae
c	10/9/96	    emax to eCont
c       12/30/96    added ice filter
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine damper
c this subroutine returns values in the array alldamp(integral) in
c the common array "filstack"
      
      include 'params.for'
      external initje
      double precision dummy, dumdamp(integral)
      integer j, k
      integer kElem
      double precision hold

*      print *, "In damper, number filter, kElem, thick " 
      do 800 k = 1, numfilter
          kElem = srcstack(k)
          hold = srcthick(k)
*          print *, k, kElem, hold
          call source(kElem, hold, dumdamp)
c          call source(scrstack(k),scrthick(k), dumdamp)
          do 900 j = 1, nInt
              srcdamp(j,k) = dumdamp(j)
c              print *, k, j, srcdamp(j,k), dumdamp(j)
  900     continue
  800 continue

      do 1000 j = 1, nInt
          dummy = one
          do 1100 k = 1, numfilter
              dummy = dummy*srcdamp(j,k)
 1100     continue
          alldamp(j) = dummy
 1000 continue

*   31 format(i8, ' ', g16.8e4,' ',g16.8e4,' ',g16.8e4,' ',g16.8e4,' ',
*     1    g16.8e4,' ', g16.8e4,' ', g16.8e4,' ', g16.8e4,' ',g16.8e4,
*     2    ' ',g16.8e4) 
*      open(30,file = 'damping.dat', form = 'formatted')
*
*      do 2000 j = 1, nInt
*          write(30,31) j, eArray(j),alldamp(j),srcdamp(j,1),
*     1          srcdamp(j,2),
*     2          srcdamp(j,3),srcdamp(j,4),srcdamp(j,5), srcdamp(j,6),
*     3          srcdamp(j,7), srcdamp(j,8)
* 2000 continue
*
*      close(30)

      return
      end              

      subroutine krasub(damping)
        
      include 'params.for'
      external initje
c Make sure this is 'params.f' in xspec!      
*      include 'params.f'
      double precision damping(nInt)

      integer i
      
      do 10 i=1,nInt
         if (eArray(i) .eq. 0) then
            damping(i)=0.0
         else 
            damping(i) = (eCont/eArray(I) -one)
c eMax changed to Econt 10/9/96
         endif
   10 continue
**      print *, 'In krasub, eMax is ', eMax
**      print *, 'In krasub, damping is ', damping
   
      return
      end
      
      subroutine linegauss(damping)
      include 'params.for'
      external initje
      
      double precision damping(nInt), eWid2, rootpi
      integer i
      
      eWid2 = eWidth*eWidth
      rootpi = one/(eWidth*sqrt(two*mypi))
            
      do 450 i = 1, nInt
          damping(i) = rootpi*
     1        exp(-(eArray(i) - eChar)**2/(two*eWidth*eWidth))
  450 continue 
**      print *, 'In linegauss, damping is ', damping
      return
      end     
      
      subroutine source(kElem, thick, damping)
      include 'params.for'
      external initje
      include 'config.inc'

      external lenact
      integer  lenact

      integer kElem
      double precision thick, damping(nInt)
      
      double precision muabs1(integral), muabs2(integral), 
     1                 muabs3(integral), muabs4(integral)
      double precision muabs(integral)
      double precision stoich(4)

      double precision elempith, cargrat, normal
      integer i

      character *256 filter, buf
      
c special nonelements:                      
c 'p1' is parylene
c 'p2' is polyimide
c 'p3' is mylar  (density wrong)
c 'p4' is teflon (polytetrafluoroethylene, density wrong)
c 'p5' is pn (ammonium dihydrogen phosphate, density wrong)
c 'p6' is pvc (polyvinyl chloride, density wrong)
c 'p7' is polypropylene
c 'ks' is the kramers continuum
c 'sy' is the synchrotron radiation spectrum
c 'gx' is the germanium quantum efficiency
c 'ax' is the fpc p10 quantum efficiency
c 'mx' is the fpc methane quantum efficiency
c 'ea' is the effective area of the mirror
c 'gn' is the gaussian
c 'px' is the pella continuum
c 'un' is the all-pass (unity) filter
     
      if (kElem.eq.parylene) then
          elempith = elemDens(kElem)*thick
          stoich(1) = 9.22579d-1
          stoich(2) = 7.7421d-2
          call newmac(cplace, muabs1)
          call newmac(hplace, muabs2)
**          print *, 'In parylene in source, thick, elempith =',
**     1			thick, elempith
          do 100 i = 1,nInt
              muabs(i) = muabs1(i)*stoich(1) + muabs2(i)*stoich(2)
              damping(i) = exp(-muabs(i)*elempith)
**           print *, i, damping(i),muabs(i), muabs1(i), muabs2(i)
  100     continue
**      print *, 'In parylene in source, damping = '
**      do 120 i = 1, nInt
**          print *, damping(i)
**  120 continue              
c stoichiometry of parylene
          
      elseif (kElem.eq.polyimide) then
          elempith = elemDens(kElem)*thick
          call newmac(cplace, muabs1)
          call newmac(hplace, muabs2)
          call newmac(oplace, muabs3)
          call newmac(nplace, muabs4)
          stoich(1) = 22.0*elemWate(cplace)
          stoich(2) = 10.0*elemWate(hplace)
          stoich(3) =  4.0*elemWate(oplace)
          stoich(4) =  2.0*elemWate(nplace)
          normal = stoich(1) + stoich(2) + stoich(3) + stoich(4)
          do 200 i = 1, nInt
              muabs(i) = (stoich(1)*muabs1(i) + stoich(2)*muabs2(i) +
     1                    stoich(3)*muabs3(i) + stoich(4)*muabs4(i))/
     2                    normal
              damping(i) = exp(-muabs(i)*elempith)
  200     continue
c stoichiometry of polyimide: C22 H10 O4 N2

      elseif (kElem.eq.polypro) then
          elempith = elemDens(kElem)*thick
          call newmac(cplace, muabs1)
          call newmac(hplace, muabs2)
          stoich(1) = elemWate(cplace)
          stoich(2) = 2.0*elemWate(hplace)
          normal = stoich(1) + stoich(2)
          do 250 i = 1, nInt
               muabs(i) = (stoich(1)*muabs1(i) + stoich(2)*muabs2(i))/
     1                     normal
               damping(i) = exp(-muabs(i)*elempith)
  250     continue

      elseif (kElem.eq.ice) then
          elempith = elemDens(kElem)*thick
          call newmac(hplace, muabs1)
          call newmac(oplace, muabs2)
          stoich(1) = 2.0*elemWate(hplace)
          stoich(2) = elemwate(oplace)
          normal = stoich(1) + stoich(2)
          do 270 i = 1, nInt
               muabs(i) = (stoich(1)*muabs1(i) + stoich(2)*muabs2(i))/
     1                     normal
               damping(i) = exp(-muabs(i)*elempith)
  270     continue
      
      elseif (kElem.eq.kramers) then
          call krasub(damping)
c Kramers continuum
          
      elseif (kElem.eq.synchro) then
c 10/9/96 addition
      bessyEnergy = eCont
* eCont already in param list
      boost = boostnorm*(bessyEnergy/bessyEnorm)
      sind = thick
c thick takes on the value of detector subtended angle
      boostsind = boost*sind
* nominally 0.236
* subtended angle added to param list as "thick" 10/9/96
      sind = boostsind/boost
      boostsind2 = boostsind*boostsind
      boostcosd = sqrt(boost*boost - boostsind*boostsind)
      tand = boostsind/boostcosd

      bessyBfield = bfield
      bessyI = current
*      bessyncline = 0.
* inclination of ecliptic should be added to param list 10/9/96
* special integration for nonzero bessyncline not implemented 10/9/96
* These values are input parameters from xspec

      bessyfluxnorm = fluxconst*(bessyI/bessyInorm)/
     1   (((bessyEnergy/bessyEnorm)**3)*(bessyBfield/bessyBnorm))
c flux norm in photons per keV^2 per second
c It is the product of 4*3^(1.5)/(20*Pi^3) and the flux per kev^2 per sec

      bessyEcrit = enercrit*
     1    ((bessyEnergy/bessyEnorm)**2)*(bessyBfield/bessyBnorm)
c synchrotron critical energy in keV

*      print *, 'In synchro '
*      print *, 'boost, sind, boostsind, boostsind2, boostcosd, tand =',
*     1          boost, sind, boostsind, boostsind2, boostcosd, tand
*
*      print *, 'bessyEnorm, bessyBnorm, bessyInorm = ',
*     1          bessyEnorm, bessyBnorm, bessyInorm
*
*      print *, 'bessyEnergy, bessyBfield, bessyI, bessyfluxnorm = ',
*     1          bessyEnergy, bessyBfield, bessyI, bessyfluxnorm
*
*      print *, 'bessyEcrit = ', bessyEcrit

      call bessyflux(damping)

*      print *, 'i, energy, y, damping = '

*      do 401 i = 1,nInt
*          print *, i, earray(i), yenergy(i), damping(i)
*  401 continue

c synchrotron radiation

      elseif (kElem.eq.ssdgerm) then
          elempith = elemDens(geplace)*thick
          call newmac(geplace, muabs)
**              print *, 'In ssdgerm,elempith,thick=',elempith,thick
          do 300 i = 1, nInt
              damping(i) = one - exp(-muabs(i)*elempith)
**              print *, i, muabs(i), damping(i)
  300     continue
  
      elseif (kElem.eq.fpcp10) then
c P10: Ar 90%, CH4 10%      
          elemDens(kElem) = (pressure/torr)*stddens*
     1                    (abzero/(abzero + temper))
          elempith = elemDens(kElem)*thick
          call newmac(cplace, muabs1)
          call newmac(hplace, muabs2)
          call newmac(arplace, muabs3)
          cargrat = one - argrat
**          stoich(1) = 0.2*cargrat*elemWate(cplace)
          stoich(1) = cargrat*elemWate(cplace)
**          stoich(2) = 0.8*cargrat*elemWate(hplace)
          stoich(2) = 4.*cargrat*elemWate(hplace)
          stoich(3) = argrat*elemWate(arplace)
c argrat is the proportional of Ar in P10          
**          normal = stoich(1) + stoich(2) + stoich(3)
          do 400 i = 1, nInt
**              muabs(i) = (stoich(1)*muabs1(i) + stoich(2)*muabs2(i)+
**     1                    stoich(3)*muabs3(i))/normal
              muabs(i) = (stoich(1)*muabs1(i) + stoich(2)*muabs2(i)+
     1                    stoich(3)*muabs3(i))
              damping(i) = one - exp(-muabs(i)*elempith)
  400     continue
          
      elseif (kElem.eq.fpcmeth) then
c methane: CH4          
          elemDens(kElem) = (pressure/torr)*stddens*
     1                    (abzero/(abzero + temper))
          elempith = elemDens(kElem)*thick
          call newmac(cplace, muabs1)
          call newmac(hplace, muabs2)
          stoich(1) = elemWate(cplace)
          stoich(2) = 4.*elemWate(hplace)
**          normal = stoich(1) + stoich(2)
          do 500 i = 1, nInt
**              muabs(i) = (stoich(1)*muabs1(i) + stoich(2)*muabs2(i))/
**     1                    normal
              muabs(i) = stoich(1)*muabs1(i) + stoich(2)*muabs2(i)
              damping(i) = one - exp(-muabs(i)*elempith)
  500     continue
          
      elseif (kElem.eq.gnplace) then
          call linegauss(damping)
          
      elseif (kElem.eq.unplace) then
c this is an "all-pass" filter      
          do 600 i = 1, nInt
              damping(i) = one
  600 continue
                    
c (old) mirror effective area

      elseif ( kElem .eq. effarea ) then

         write( buf, 800 ) int(effarea)
 800     format( 'jmkmod: setting filter = ', i2,
     $     ' to get effective areas is no longer supported;' )
         call xwrite( buf, 1 )
         call xwrite( 'jmkmod: ==> use an external filter', 1 )

         do 801 i = 1, nInt
            damping(i) = one
 801     continue


c external filter read from file in default table directory
      elseif (kElem .ge. fltdflt .and. kElem .lt. fltfile ) then

         write( filter, fltfmt )  kElem - fltdflt
         call jmkflt( damping, fltdir // filter, thick )
          
c external filter read from file in current directory
      elseif (kElem .ge. fltfile .and. kElem .lt. fltenv ) then
          
         write( filter, fltfmt )  kElem - fltfile
         call jmkflt( damping, filter, thick )

c external filter read from filename in environmental variable filter_NNN
      elseif (kElem .ge. fltenv ) then

         write( filter, fltenvfmt )  kElem - fltenv
         call xwrite( 'jmkmod: reading environmental variable ' //
     $        filter, 11 )
         call getenv( filter, buf )
         if ( buf .ne. ' ' ) then
            call jmkflt( damping, buf, thick )
         else
            call xwrite( 'jmkmod: environmental variable ' //
     $           filter(:lenact(filter)) // ' doesn''t exist', 1 )
          do 80 i = 1, nInt
             damping(i) = one
 80       continue
         endif
      else
          elempith = elemDens(kElem)*thick
*      print *, "In source, elemDens ", elemDens
*      print *, "In source, elemWate ", elemWate
*      print *, "In souce, elemThick ", elemThick
*      print *, "In source ",kElem,elemDens(kElem),elempith,thick

*          print *, "damping in source = "
          call newmac(kElem, muabs)
          do 15 i = 1, nInt
              damping(i) = exp(-muabs(i)*elempith)
*              print *, i, evArr(i), eArray(i), damping(i), muabs(i)
   15     continue
c the default case
      endif

      return
      
      end          
      
      subroutine winfpc(thickAL, thickPOLY, damping)
      include 'params.for'
      external initje
      
      double precision thickAL, thickPOLY, damping(nInt)
      double precision dampPOLY(integral), dampAL(integral)
      
      integer i
      
      call source(polyimide, thickPOLY, dampPOLY)
      call source(alplace, thickAL, dampAL)
      
      do 90 i = 1, nInt
          damping(nInt) = dampPOLY(nInt)*dampAL(nInt)
   90 continue
      return
      end
      
      subroutine winssd(thickAL, thickPARY, damping)
      include 'params.for'
      external initje
      
      double precision thickAL, thickPARY, damping(nInt)
      double precision dampPARY(integral), dampAL(integral)
      
      integer i
      
      call source(parylene, thickPARY, dampPARY)
      call source(alplace, thickAL, dampAL)
      
      do 90 i = 1, nInt
          damping(nInt) = dampPARY(nInt)*dampAL(nInt)
   90 continue
      end    
      
